{#
SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
SPDX-FileCopyrightText: 2024 Arjen Hiemstra <ahiemstra@heimr.nl>

This is the template that is used to generate CMakeLists for tests.

REUSE-IgnoreStart
#}
# SPDX-License-Identifier: BSD-2-Clause
# SPDX-FileCopyrightText: 2024 Arjen Hiemstra <ahiemstra@heimr.nl>
#
# This file is automatically generated from CMakeLists.tests.txt.j2
# To regenerate, run `tools/propertygenerator/generate_properties.py`.
{# REUSE-IgnoreEnd #}

ecm_add_tests(
{% for type in types %}
    Test{{ type.type_name | ucfirst }}.cpp
{% endfor %}
    LINK_LIBRARIES Qt6::Test Union::Union
)
