{#
SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
SPDX-FileCopyrightText: 2024 Arjen Hiemstra <ahiemstra@heimr.nl>

This is the template that is used to generate CMakeLists for property classes.

REUSE-IgnoreStart
#}
# SPDX-License-Identifier: BSD-2-Clause
# SPDX-FileCopyrightText: 2024 Arjen Hiemstra <ahiemstra@heimr.nl>
#
# This file is automatically generated from CMakeLists.txt.j2
# To regenerate, run `tools/propertygenerator/generate_properties.py`.
{# REUSE-IgnoreEnd #}

target_sources({{ target_name }} PRIVATE
{% for type in types %}
    {{ type.type_name }}{{ file_suffix }}.cpp
    {{ type.type_name }}{{ file_suffix }}.h
{% endfor %}
)
