// SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
// SPDX-FileCopyrightText: 2025 Martin Sh <hemisputnik@proton.me>

#include "widgetpage.h"

WidgetPage::WidgetPage(QWidget *parent)
    : QWidget(parent)
{
}

#include "moc_widgetpage.cpp"
