/*
 * SPDX-License-Identifier: BSD-2-Clause
 * SPDX-FileCopyrightText: 2025 Arjen Hiemstra <ahiemstra@heimr.nl>
 */

/*
 * Base style definition. Don't put any rules here, place them in a separate
 * file and use @import to import it.
 */

@import "variables.css";

@import "defaults.css";

@import "button.css";
@import "delegate.css";
@import "text.css";
@import "check.css";
@import "popup.css";
@import "application.css";
@import "bars.css";
@import "kirigami.css";
@import "tables.css";
@import "calendar.css";