// SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
// SPDX-FileCopyrightText: 2024 Arjen Hiemstra <ahiemstra@heimr.nl>

#include "InputPlugin.h"

using namespace Union;

InputPlugin::InputPlugin(QObject *parent)
    : Plugin(parent)
{
}

#include "moc_InputPlugin.cpp"
