/*
 * SPDX-License-Identifier: BSD-2-Clause
 * SPDX-FileCopyrightText: 2026 Akseli Lahtinen <akselmo@akselmo.dev>
 */

monthgrid, dayofweekrow, weeknumbercolumn  {
	background-color: var(--view-background-color);
	border: 1px solid var(--view-border-color);
	& > label {
		outline: 1px solid var(--view-border-color);
		text-alignment-horizontal: center;
		text-alignment-vertical: center;
		color: var(--view-text-color);
		background-color: var(--view-disabled-background-color);
	}
}

monthgrid label {
	color: var(--view-disabled-text-color);
	&.current-month {
		color: var(--view-text-color);
	}

	&.today {
		background-color: var(--highlight-color);
		color: var(--highlight-text-color);
	}
}
